#include <fstream.h>
#include "parse.h"
#include "getvalue.h"
#include "version.h"

char *helpstring="Insert more help here";

int main(int argc, char *argv[]) {
  // parsing command line. See getvalue.hh for details;
  char *str1filename="str1.out";
  char *str2filename="str2.out";
  Real f=0.5;
  Real df=0.0;
  int sigdig=5;
  int dohelp=0;
  int dummy=0;
  AskStruct options[]={
    {"","STRucture PATH generator " MAPS_VERSION ", by Axel van de Walle",TITLEVAL,NULL},
    {"-s1","Input file defining structure 1 (default: str1.out)",STRINGVAL,&str1filename},
    {"-s2","Input file defining structure 2 (default: str2.out)",STRINGVAL,&str2filename},
    {"-f","Interpolation fraction in [0,1]",REALVAL,&f},
    {"-df","Dimer distance [0,1]",REALVAL,&df},
    {"-sig","Number of significant digits printed (default: 5)",INTVAL,&sigdig},
    {"-h","Display more help",BOOLVAL,&dohelp},
    {"-d","Use all default values",BOOLVAL,&dummy}
  };
  if (!get_values(argc,argv,countof(options),options)) {
    display_help(countof(options),options);
    return 1;
  }
  if (dohelp) {
    cout << helpstring;
    return 1;
  }

  cout.setf(ios::fixed);
  cout.precision(sigdig);

  // parsing lattice and structure files. See parse.hh for detail;
  Structure str1;
  Array<Arrayint> labellookup;
  Array<AutoString> label;
  rMatrix3d axes;
  {
    ifstream str1file(str1filename);
    if (!str1file) ERRORQUIT("Unable to open structure 1 file");
    parse_lattice_file(&str1.cell, &str1.atom_pos, &str1.atom_type, &labellookup, &label, str1file, &axes);
    wrap_inside_cell(&str1.atom_pos,str1.atom_pos,str1.cell);
    fix_atom_type(&str1, labellookup);
  }
  Structure str2;
  {
    ifstream str2file(str2filename);
    if (!str2file) ERRORQUIT("Unable to open structure 2 file");
    parse_structure_file(&str2.cell, &str2.atom_pos, &str2.atom_type, label, str2file, NULL);
    wrap_inside_cell(&str2.atom_pos,str2.atom_pos,str2.cell);
  }

  Structure str;
  str.cell=(1-f)*str1.cell+f*str2.cell;
  str.atom_type=str1.atom_type;
  str.atom_pos.resize(str1.atom_pos.get_size());
  Array<rVector3d> dir;
  dir.resize(str1.atom_pos.get_size());
  Array<rVector3d> frac1;
  Array<rVector3d> frac2;
  rMatrix3d icell1=!str1.cell;
  rMatrix3d icell2=!str2.cell;
  for (int at=0; at<str1.atom_pos.get_size(); at++) {
    rVector3d frac1=icell1*str1.atom_pos(at);
    rVector3d frac2=icell2*str2.atom_pos(at);
    rVector3d cellsh=round(frac2-frac1);
    frac2=frac2-cellsh;
    str.atom_pos(at)=str.cell*((1.-f)*frac1+f*frac2);
    dir(at)=str.cell*(frac2-frac1)*df;
  }

  write_structure(str,label,axes,cout,0);
  if (df>0) {
    for (int at=0; at<dir.get_size(); at++) {
      cout << (!axes)*dir(at) << " ZZdimer" << endl;
    }
  }
}
